"""
Auto-generated literal type definitions for tailwind-merge.
Generated from types.ts
"""

from typing import Literal

# Theme group IDs from the original TypeScript implementation
DefaultThemeGroupIds = Literal[
    'animate', 'aspect', 'blur', 'breakpoint', 'color', 'container', 'drop-shadow', 'ease', 
    'font-weight', 'font', 'inset-shadow', 'leading', 'perspective', 'radius', 'shadow', 'spacing', 
    'text', 'tracking'
]

# Class group IDs from the original TypeScript implementation
DefaultClassGroupIds = Literal[
    'accent', 'align-content', 'align-items', 'align-self', 'animate', 'appearance', 'aspect', 
    'auto-cols', 'auto-rows', 'backdrop-blur', 'backdrop-brightness', 'backdrop-contrast', 
    'backdrop-filter', 'backdrop-grayscale', 'backdrop-hue-rotate', 'backdrop-invert', 
    'backdrop-opacity', 'backdrop-saturate', 'backdrop-sepia', 'backface', 'basis', 
    'bg-attachment', 'bg-blend', 'bg-clip', 'bg-color', 'bg-image', 'bg-origin', 'bg-position', 
    'bg-repeat', 'bg-size', 'blur', 'border-collapse', 'border-color-b', 'border-color-e', 
    'border-color-l', 'border-color-r', 'border-color-s', 'border-color-t', 'border-color-x', 
    'border-color-y', 'border-color', 'border-spacing-x', 'border-spacing-y', 'border-spacing', 
    'border-style', 'border-w-b', 'border-w-e', 'border-w-l', 'border-w-r', 'border-w-s', 
    'border-w-t', 'border-w-x', 'border-w-y', 'border-w', 'bottom', 'box-decoration', 'box', 
    'break-after', 'break-before', 'break-inside', 'break', 'brightness', 'caption', 'caret-color', 
    'clear', 'col-end', 'col-start-end', 'col-start', 'color-scheme', 'columns', 'container', 
    'content', 'contrast', 'cursor', 'delay', 'display', 'divide-color', 'divide-style', 
    'divide-x-reverse', 'divide-x', 'divide-y-reverse', 'divide-y', 'drop-shadow', 'duration', 
    'ease', 'end', 'field-sizing', 'fill', 'filter', 'flex-direction', 'flex-wrap', 'flex', 
    'float', 'font-family', 'font-size', 'font-smoothing', 'font-stretch', 'font-style', 
    'font-weight', 'forced-color-adjust', 'fvn-figure', 'fvn-fraction', 'fvn-normal', 
    'fvn-ordinal', 'fvn-slashed-zero', 'fvn-spacing', 'gap-x', 'gap-y', 'gap', 'gradient-from-pos', 
    'gradient-from', 'gradient-to-pos', 'gradient-to', 'gradient-via-pos', 'gradient-via', 
    'grayscale', 'grid-cols', 'grid-flow', 'grid-rows', 'grow', 'h', 'hue-rotate', 'hyphens', 
    'indent', 'inset-ring-color', 'inset-ring-w', 'inset-shadow-color', 'inset-shadow', 'inset-x', 
    'inset-y', 'inset', 'invert', 'isolation', 'justify-content', 'justify-items', 'justify-self', 
    'leading', 'left', 'line-clamp', 'list-image', 'list-style-position', 'list-style-type', 'm', 
    'max-h', 'max-w', 'mb', 'me', 'min-h', 'min-w', 'mix-blend', 'ml', 'mr', 'ms', 'mt', 'mx', 
    'my', 'object-fit', 'object-position', 'opacity', 'order', 'outline-color', 'outline-offset', 
    'outline-style', 'outline-w', 'overflow-x', 'overflow-y', 'overflow', 'overscroll-x', 
    'overscroll-y', 'overscroll', 'p', 'pb', 'pe', 'perspective-origin', 'perspective', 'pl', 
    'place-content', 'place-items', 'place-self', 'placeholder-color', 'pointer-events', 
    'position', 'pr', 'ps', 'pt', 'px', 'py', 'resize', 'right', 'ring-color', 'ring-offset-color', 
    'ring-offset-w', 'ring-w-inset', 'ring-w', 'rotate-x', 'rotate-y', 'rotate-z', 'rotate', 
    'rounded-b', 'rounded-bl', 'rounded-br', 'rounded-e', 'rounded-ee', 'rounded-es', 'rounded-l', 
    'rounded-r', 'rounded-s', 'rounded-se', 'rounded-ss', 'rounded-t', 'rounded-tl', 'rounded-tr', 
    'rounded', 'row-end', 'row-start-end', 'row-start', 'saturate', 'scale-3d', 'scale-x', 
    'scale-y', 'scale-z', 'scale', 'scroll-behavior', 'scroll-m', 'scroll-mb', 'scroll-me', 
    'scroll-ml', 'scroll-mr', 'scroll-ms', 'scroll-mt', 'scroll-mx', 'scroll-my', 'scroll-p', 
    'scroll-pb', 'scroll-pe', 'scroll-pl', 'scroll-pr', 'scroll-ps', 'scroll-pt', 'scroll-px', 
    'scroll-py', 'select', 'sepia', 'shadow-color', 'shadow', 'shrink', 'size', 'skew-x', 'skew-y', 
    'skew', 'snap-align', 'snap-stop', 'snap-strictness', 'snap-type', 'space-x-reverse', 
    'space-x', 'space-y-reverse', 'space-y', 'sr', 'start', 'stroke-w', 'stroke', 'table-layout', 
    'text-alignment', 'text-color', 'text-decoration-color', 'text-decoration-style', 
    'text-decoration-thickness', 'text-decoration', 'text-overflow', 'text-transform', 'text-wrap', 
    'top', 'touch-pz', 'touch-x', 'touch-y', 'touch', 'tracking', 'transform-origin', 
    'transform-style', 'transform', 'transition-behavior', 'transition', 'translate-none', 
    'translate-x', 'translate-y', 'translate-z', 'translate', 'underline-offset', 'vertical-align', 
    'visibility', 'w', 'whitespace', 'will-change', 'z'
]
